# Alexa Elterngeld Demonstrator
https://www.oeffentliche-it.de/werkstatt/alexa-elterngeld

## Mit Alexa durch den Antragsdschungel
Im Rahmen der ÖFIT-Werkstatt wurde ein Demonstrator entwickelt, der am Beispiel Elterngeld veranschaulicht, wie Sprachassistenten zur Antragstellung eingesetzt werden können.  
Mit nur wenigen Abfragen kann so ein vollständiger Antrag gestellt werden.

Dabei wird deutlich, dass Antragsprozesse nicht nur eins zu eins digitalisiert werden sollten. Mit intelligenter Dialogführung können Anträge wesentlich vereinfacht werden.  
Der Demonstrator nimmt auch Bezug zu aktuellen Entwicklungen im E-Government.  
Ansätze wie Once Only und standardisierte Servicekonten können helfen Antragsverfahren weiter zu vereinfachen.

## Ordner-Struktur
Dieses Repository enthält alle Dateien um einen eigenen Skill (im Entwicklermodus) auf einem Alexa-kompatiblen Gerät ausprobieren zu können.

### lambda
Beinhaltet die Python-Implementierung des Zustandsautomaten als Lambda-Funktion, welche vom Skill genutzt wird.  
Mehr Informationen zur Einrichtung findet sich im Amazon Entwicklerhandbuch unter [Erste Schritte mit AWS Lambda](https://docs.aws.amazon.com/de_de/lambda/latest/dg/getting-started.html)

### ask
Die JSON-Datei für die [Alexa Skill Kit Konsole](https://developer.amazon.com/alexa/console/ask/) beinhaltet:
*  Startname des Skills
*  Intents mit Slots sowie
*  Utterances

### doc
Hier befindet sich die Frage-Antwort-Kaskade als Word-Tabelle.  
Sie ist die Basis für die Implementierung der Zustandsübergänge innerhalb der Lambda-Funktion im nächsten Abschnitt.

## Getting started
* [AWS-Umgebung](https://aws.amazon.com): Loggen Sie sich mit einem AWS-Account ein unter https://aws.amazon.com
  * Wechseln Sie in den Bereich [AWS DynamoDB](https://aws.amazon.com/de/dynamodb/)
    * Erstellen Sie mit der [AWS DynamoDB Console](https://console.aws.amazon.com/dynamodb/home) eine Tabelle
      * Tabellenname `elterngeld`
      * Primärer Partitionsschlüssel `account` (Zeichenfolge)
    * Notieren Sie sich den Amazon Resource Name (ARN), z.B.:	`arn:aws:dynamodb:eu-west-1:01234567890:table/elterngeld`

  * [Identity and Access Management (IAM)](https://docs.aws.amazon.com/de_de/IAM/latest/UserGuide/introduction.html)
    * Erstellen sie mit der [IAM Console](https://console.aws.amazon.com/iam/home#/home) eine Rolle mit Namen `alexa`
      * Erstellen Sie in der Rolle eine Richtlinie `simpleDynamoDBAccess` für den Service `DynamoDB` mit Zugriffsebene `Begrenzt: Lesen, Schreiben` für `Alle Ressourcen`

  * [AWS Lambda](https://aws.amazon.com/lambda/)-Funktion
    * Erstellen Sie  mit der [AWS Lambda Console](https://console.aws.amazon.com/lambda/home) eine Lambda-Funktion mit dem Namen `Elternzeit`
    * Weisen sie der Funktion die Ausführungsrolle `alexa` zu
    * Handler: `lambda_function.lambda_handler`
    * Laufzeit: Python 3.6
    * Verwenden Sie den Inline Code Editor um den Inhalt der Datei `lambda/lambda_function.py` einzufügen
    * Notieren Sie sich den Amazon Resource Name (ARN), z.B.: `arn:aws:lambda:eu-west-1:01234567890:function:Elternzeit`

* [Amazon-Entwicklerumgebung](https://developer.amazon.com/alexa): Loggen Sie sich mit einem Developer-Account ein unter https://developer.amazon.com/alexa
  * Erstellen Sie mit der [Alexa Skill Kit ASK](https://developer.amazon.com/alexa/console/ask) einen neuen Skill `Elterngeld`
  * Fügen sie den Inhalt der Datei `ask/skill.json` mittels JSON Editor ein
  * Klicken sie auf `Save Model` und anschließend auf `Build Model`
  * Tragen sie den ARN der Lambda-Funktion unter Endpoints `AWS Lambda ARN` -> `Default Region` ein

* [Alexa Console](https://alexa.amazon.com):  Loggen Sie sich mit dem Amazon-Account Ihrer Amazon-Alexa ein unter https://alexa.amazon.com oder nutzen Sie die Alexa App
    * Aktivieren Sie den Skill unter `Skills` -> `Ihre Skills` -> `Entwicklerskills`
