from __future__ import print_function
from datetime import datetime
import locale
import calendar
import boto3
locale.setlocale(locale.LC_ALL, 'de_DE.UTF-8')

# --------------- global variables ----------------

debug = False
nr_no_realise = 1

client = boto3.resource('dynamodb')

# keys: startkm, endkm, mcount
mez = {}
fez = {}

months = ["januar", "februar", "märz", "april", "mai", "juni", "juli", "august", "september", "oktober", "november", "dezember"]
months2n = {"januar":1, "februar":2, "märz":3, "april":4, "mai":5, "juni":6, "juli":7, "august":8, "september":9, "oktober":10, "november":11, "dezember":12}

askboth = False
askmother = False

last_reprompt_text = ""
# --------------- Toolset, which gets used inside the behaviour functions-------
def norealise(session_attributes, card_title, debug_txt, err_txt):
#   =========
    global nr_no_realise
    global last_reprompt_text
 
    if err_txt == "":        
        if nr_no_realise == 1:
            err_txt = " Sorry, das habe ich nicht richtig verstanden. "
        elif nr_no_realise == 2:
            err_txt = " Können Sie bitte ihre Antwort wiederholen? "
        elif nr_no_realise == 3:
            err_txt = " Wie bitte? "
        elif nr_no_realise == 4:
            err_txt = " Was meinen Sie? "
        elif nr_no_realise == 5:
            err_txt = " Daraus kann ich mir keinen Reim machen. "
        elif nr_no_realise == 6:
            err_txt = " Das erschließt sich mir nicht. "
        elif nr_no_realise == 7:
            err_txt = " Ich verstehe nur Bahnhof. "
        elif nr_no_realise == 8:
            err_txt = " Das war mir nicht verständlich. "
        else: 
            err_txt = " Können Sie das nocheinmal sagen? "
            nr_no_realise = 0
        nr_no_realise = nr_no_realise + 1
    if debug:
        speech_output = debug_txt + err_txt + last_reprompt_text
    else:
        speech_output = err_txt + last_reprompt_text 
    
    should_end_session = False
    return build_response(session_attributes, build_speechlet_response(
        card_title, speech_output, last_reprompt_text, should_end_session))

def compute_duration(start,end):
#   ================
    if end <start:
        duration=13-start+end
    elif start==end:
        duration=1
    else:
        duration=end-start+1
    return duration
    
def date2str(date):
#   ========
    return str(date.day) + "." + str(date.month) + "." + str(date.year)
    
def get_birthdate():
#   =============
    today = datetime.now()
    year = today.year
    month = today.month
    day = today.day
    if day-8 < 1:
        month = month - 1
        if month < 1:
            month = 12
            year = year - 1
            day = 23 + day
        elif month == 2 and year % 4 == 0:
            day = 21 + day
        elif month == 2:
            day = 20 + day
        elif month==1 or month==3 or month==5 or month==7 or month==8 or month==10:
            day = 23 + day
        else:
            day = 22 + day
    else:
        day = day - 8
    # return today + datetime.timedelta(days=8)
    return datetime.date(datetime(year,month,day))
    
def get_first_lifemonth(birthdate):
#   ===================
    return get_end_lifemonth_n(birthdate, 1)

def get_start_lifemonth_n(birthdate, n):
#   =====================
    if n < 1:
        n = 1
    year = birthdate.year
    month = birthdate.month + n - 1
    while month > 12:
        month = month - 12
        year = year + 1
    day = birthdate.day
    if day == 31:
        if month == 2 and year % 4 == 0:
            day = 29
        elif month == 2:
            day = 28
        elif month==4 or month==6 or month==9 or month==11:
            day = 30
    elif day > 28:
        if month == 2 and year % 4 == 0:
            day = 29
        elif month == 2:
            day = 28
    return datetime.date(datetime(year,month,day))
    
def get_end_lifemonth_n(birthdate, n):
#   ===================
    if n < 1:
        n = 1
    year = birthdate.year
    month = birthdate.month + n
    day = birthdate.day
    if day == 1:
        month = month - 1
        while month > 12:
            month = month - 12
            year = year + 1
        if month == 2 and year % 4 == 0:
            day = 29
        elif month == 2:
            day = 28
        elif month==1 or month==3 or month==5 or month==7 or month==8 or month==10:
            day = 31
        else:
            day = 30
    else:
        day = day - 1
        while month > 12:
            month =  month - 12
            year = year + 1
        if month == 2 and year % 4 == 0 and day > 29:
            day = 29
        elif month == 2 and day > 28:
            day = 28
    return datetime.date(datetime(year,month,day))
    

# --------------- Helpers that build all of the responses ----------------------

def build_speechlet_response(title, output, reprompt_text, should_end_session):
#   ========================
    global last_reprompt_text
    last_reprompt_text = reprompt_text
 
    return {
        'outputSpeech': {
            'type': 'PlainText',
            'text': output
        },
        'card': {
            'type': 'Simple',
            'title': "SessionSpeechlet - " + title,
            'content': "SessionSpeechlet - " + output
        },
        'reprompt': {
            'outputSpeech': {
                'type': 'PlainText',
                'text': reprompt_text
            }
        },
        'shouldEndSession': should_end_session
    }

def build_response(session_attributes, speechlet_response):
#   ==============
    return {
        'version': '1.0',
        'sessionAttributes': session_attributes,
        'response': speechlet_response
    }

def clearDB(session):
    user= session['user']['userId']  # get the userId of the current user
    user=user[18:] # we need the string after the 19th character.

    table = client.Table('elterngeld') # DynamoDB Table Name
    table.put_item(Item={'account':user,'iteration':0, 'mez': {}, 'fez': {}, 'askboth': False, 'askmother': False})


# --------------- Functions that control the skill's behavior ------------------

        
def welcome(session):
#   =======
    global last_reprompt_text
    global mez, fez, askboth, askmother
    
    table = client.Table('elterngeld') # DynamoDB Table Name
    user = session['user']['userId']  # get the userId of the current user
    user = user[18:] # we need the string after the 19th 
    # 'account':user,'iteration':seatit, 'lasttext': last_reprompt_text, 'mez': mez, 'fez': fez, 'askboth': askboth, 'askmother': askmother

    # 'account':user,'iteration':seatit, 'lasttext': last_reprompt_text, 'mez': mez, 'fez': fez}
    warPause = True
    seatit = 0
    try:
        response = table.get_item(Key={ 'account': user })
        if 'Item' in response:
            seatit = int(response['Item']['iteration'])
            if seatit != 0:
                last_reprompt_text = response['Item']['lasttext']
                mez = response['Item']['mez']
                fez = response['Item']['fez']
                askboth = response['Item']['askboth']
                askmother = response['Item']['askmother']
            else:
                warPause = False
        else:
            warPause = False
    except:
        warPause = False
        
    card_title = "welcome"
    if warPause:
        session_attributes = {'iteration': seatit}
        speech_output = "OK. " + last_reprompt_text
        reprompt_text = last_reprompt_text
    else:
        mez = {}
        fez = {}
        session_attributes = {'iteration':0}
        speech_output = "Herzlich willkommen beim Antragsassistenten für Elterngeld. Bevor wir loslegen, soll ich ihnen zunächst ausführliche Informationen zu Elterngeld und Elternzeit geben?"
#        speech_output = "Ich werde Ihnen Fragen stellen, um die Anträge für Basis-Elterngeld, und Elternzeit zu beantragen. Ich kann auch Fragen wiederholen. Können Sie eine Frage nicht sofort beantworten, sagen Sie bitte, Pause. Mit dem Kommando, Computer Elterngeld, geht es dann weiter. Mit dem Kommando, Stopp, wird alles beendet. Brauchen Sie noch weitere Informationen zu den Zusammenhang zwischen Lebensmonaten des Kindes, Elternzeit und Elterngeld?"
#        speech_output = "Ich kann dabei helfen, mit ihnen gemeinsam die Anträge für Basis Elterngeld beim Bürgeramt und für Elternzeit beim Arbeitgeber zu erstellen. Ich werde nach der Elternzeit für die Lebensmonate des Kindes Fragen, für die Sie Elterngeld beantragen möchten. Brauchen sie noch weitere Informationen bevor wir gemeinsam die Anträge erstellen?"
        reprompt_text = "Brauchen sie noch weitere Informationen bevor wir gemeinsam die Anträge erstellen?"
#    speech_output = "Um Elterngeld vom Staat zu erhalten müssen Sie zuerst Elternzeit beim Arbeitgeber beantragen. Ich kann für Sie die Anträge stellen. Dafür benötige ich noch einige Informationen von Ihnen. Sind Sie damit einverstanden?"
#    reprompt_text = "Sind Sie damit einverstanden, dass ich mit ihren Informationen die Anträge stelle?"
    
    should_end_session = False
    return build_response(session_attributes, build_speechlet_response(
        card_title, speech_output, reprompt_text, should_end_session))
        
        
def part_start(session,intent):
#   ==========
    global askboth
    global last_reprompt_text
    
    card_title = "part_start"
    should_end_session = False
    
    speech_output = "Fehler " + str(session['attributes']['iteration']) + "im start Teil durch [" + intent['name'] + "]"
    reprompt_text = speech_output
    session_attributes = session['attributes']
    seatit = session['attributes']['iteration']
    
    if seatit == 0:
        birthdate=get_birthdate()
        mend = birthdate.day-1
        strmend = str(mend) +". Tag "
        if mend == 0:
            strmend = "Monatsende"
        speech_output = "Sie sind beide Gehaltsempfänger. In ihrem Fall würde die Mutter den Maximalbetrag von 1800 Euro, und der Vater würde 1495 Euro, Basis-Elterngeld, pro Monat erhalten. Zusätzlich zu den 2 Monaten Mutterschutz können beide Partner zusammen noch 12 Monate lang Basis-Elterngeld bekommen, die Mutter alleine nur 10 Monate. Elterngeld gibt es nicht pro Kalendermonat, sondern für mindestens zwei Lebensmonate des Kindes. Lebensmonat meint: Ihr Kind ist am " + date2str(birthdate) + " geboren. Die Elternzeit sollte daher am " + str(birthdate.day) + ". Tag eines Monats beginnen, und am " +  strmend + " eines nachfolgenden Monats enden. War das zu viel Information, kann ich das gerne wiederholen. Habe ich ihnen die Zusammenhänge ausreichend erläutert?"
        reprompt_text = "Habe ich Ihnen den Sachverhalt deutlich machen können?"
#        speech_output = "Beide Partner können zusammen 14 Monate Basis Elterngeld bekommen, einer nur 12 Monate. Elterngeld gibt es nicht Monatsweise, sondern nur für die Lebensmonate des Kindes. Die Elternzeit sollte daher am " + str(birthdate.day) + ". Tag eines Monats beginnen und am " +  strmend + " eines Nachfolgemonats enden. Ich werde nur nach der Elternzeit für das Elterngeld fragen und diese zeitgleich zu den Lebensmonaten des Kindes beantragen. Haben Sie das verstanden?"
#        reprompt_text = "Haben Sie das verstanden, dass es Elterngeld für die Lebensmonate des Kindes gibt?"
#        speech_output = "Elterngeld gibt es nicht monatsweise, sondern für die Lebensmonate des Kindes. Ich werde nur nach der Elternzeit für das Elterngeld fragen und diese zeitgleich zu den Lebensmonaten des Kindes beantragen. Haben Sie das verstanden?"
#        reprompt_text = "Haben Sie das verstanden, dass es Elterngeld für die Lebensmonate des Kindes gibt?"
        session_attributes = {'iteration':1}
    elif seatit == 1:           # "Verstanden"
        speech_output = "Möchten Sie beide, nur der Vater oder nur die Mutter Elternzeit nehmen?"
        reprompt_text = speech_output 
        askboth = False
        session_attributes = {'iteration':2}
    elif seatit == 2:           # "Einer"
        speech_output = "Sie können das Basis Elterngeld voll ausschöpfen, wenn der zweite Partner mindestens zwei Monate in Elternzeit geht. Möchte trotzdem nur einer Elternzeit nehmen?"
        reprompt_text = "Möchte trotzdem nur einer Elternzeit nehmen?" 
        session_attributes = {'iteration':3}
    elif seatit == 3:           # "Wirklich nur einer"
        speech_output = "Möchte der Vater oder die Mutter Elternzeit nehmen?"
        reprompt_text = "Möchte die Mutter oder der Vater Elternzeit nehmen?" 
        session_attributes = {'iteration':4}
    elif seatit == 4:
        session_attributes = {'iteration':200}
        return part_two(session, intent)
    elif seatit == 5:
        speech_output = "Sie sind beide Gehaltsemfänger und habt eine Vollzeit Arbeitsverhältnis. Die Mutter würde den Maximalbetrag von 1800 Euro bekommen und der Vater würde 1495 Euro Elterngeld erhalten. Mit einigen wenigen weiteren Informationen kann ich für Euch die Anträge stellen. Kann ich diese Informationen erfragen?"
        reprompt_text = "Möchte die Mutter oder der Vater Elternzeit nehmen?" 
        session_attributes = {'iteration':6}
    elif seatit == 6:
        speech_output = "Ich muss wissen, ob beide oder nur einer von ihnen Elternzeit nimmt und die genauen Zeiträume für diese Elternzeit. Für diese Zeit kann ich Elternzeit und auch Elterngeld beantragen. Ist es in Ordnung, wenn ich diese Informationen erfrage?"
        reprompt_text = "Ist es in Ordnung, wenn ich diese genauen Monate für diese Elternzeit erfrage?" 
        session_attributes = {'iteration':7}
    elif seatit == 7:
        birthdate=get_birthdate()
        lifemonth=get_first_lifemonth(birthdate)
        mend = birthdate.day-1
        strmend = str(mend) +". Tag "
        if mend == 0:
            strmend = "Monatsende"
        speech_output = "Ich werde nach der Elternzeit für die Lebensmonate des Kindes fragen, für die Sie Basis Elterngeld beantragen möchten. Ihr Kind ist am " + date2str(birthdate) + " geboren. Der erste Lebensmonat geht vom " +  date2str(birthdate) + " bis zum " + date2str(lifemonth) + "." + " Die Elternzeit sollte daher am " + str(birthdate.day) + ". Tag eines Monats beginnen und am " +  strmend + " eines Nachfolgemonats enden. Um Elterngeld zu erhalten muss man mindestens 2 Lebensmonate Elternzeit nehmen. Habe ich Ihnen den Sachverhalt deutlich machen können?"
        #speech_output = "Antwort 7 Teil Start"
        reprompt_text = "Habe ich ihnen die Zusammenhänge ausreichend erläutert?"
        session_attributes = {'iteration':8}
    else:
        session_attributes = {'iteration': 3}
        last_reprompt_text = " Möchte nur eine von Ihnen Elternzeit nehmen? "
        return norealise(session_attributes, card_title, speech_output, "")
        
    return build_response(session_attributes, build_speechlet_response(
        card_title, speech_output, reprompt_text, should_end_session))
 
def part_one(session,intent):
#   ========
    # Beide Eltern möchten Elternzeit nehmen
    global askboth
    global askmother
    global mez
    global fez
    global last_reprompt_text
    
    card_title = "part_one"
    should_end_session = False
    speech_output = "Fehler  " + str(session['attributes']['iteration']) + " in Teil Eins durch [" + intent['name'] + "]"
    reprompt_text = speech_output
    
    session_attributes = session['attributes']
    seatit = session['attributes']['iteration']
    
    if seatit == 100:
        speech_output = "Um die Betreuungszeit zu teilen nimmt bei vielen Eltern die Mutter nach den zwei Monaten Mutterschutz noch fünf Monate Elternzeit und danach der Vater sieben Monate Elternzeit. Möchten Sie auch so die Betreuungszeit für ihr Kind organisieren?"
        reprompt_text = "Möchte die Mutter 5 Monate und danach der Vater 7 Monate Elternzeit nehmen?"
        askboth = True
        session_attributes = {'iteration': 101}
    elif seatit == 102:
        bday = get_birthdate()
        mstart = mez['startkm']
        startkm = get_start_lifemonth_n(bday, mez['startkm'])
        endkm = get_end_lifemonth_n(bday, mez['endkm'])
        mcount = mez['mcount']

        
        fmstart = fez['startkm']
        fstartkm = get_start_lifemonth_n(bday, fez['startkm'])
        fendkm = get_end_lifemonth_n(bday, fez['endkm'])
        fmcount = fez['mcount']
                
        if mcount < 2:
            seatit =  103
        elif mcount > 10:
            seatit = 104
        elif mstart < 3:
            seatit = 105
        elif fmcount < 2:
            seatit = 106
        elif fmcount > 12:
            seatit = 107
        elif fmcount + mcount > 12:
            seatit = 108
        else:
            session['attributes']['iteration'] = 111
            seatit = 111
            #return part_one(session, intent)
    bday = get_birthdate()
    startkm = get_start_lifemonth_n(bday, 3)
    if seatit == 103: 
        # 103
        speech_output = "Um Elterngeld zu erhalten muss die Mutter nach dem Mutterschutz, also ab dem " + date2str(startkm) + ", mindestens 2 Lebensmonate Elternzeit nehmen. Wieviel Monate möchte die Mutter in Elternzeit gehen?"
        reprompt_text = "Wieviel Monate möchte die Mutter in Elternzeit gehen?" 
        askmother = True
        session_attributes = {'iteration':104}
    elif seatit == 104: 
        # 104
        mcount = mez['mcount']
         
        speech_output = "Die Mutter kann beim Basis Elterngeld nach dem Mutterschutz, also ab dem " + date2str(startkm) + ", maximal 10 Monate und nicht für " + str(mcount) + " Monate Elterngeld beantragen. Von welchen Monat bis welchen Monat möchte die Mutter in Elternzeit gehen?"
        reprompt_text = "Von welchen Monat bis welchen Monat möchte die Mutter in Elternzeit gehen?" 
        askmother = True
        session_attributes = {'iteration':105}
    elif seatit == 105: 
        # 105
        abstr =  " ab dem ersten Lebensmonat, "
        if mstart == 1:
            abstr = " ab der Geburt,"
        bday = get_birthdate()
        
        startkm = get_start_lifemonth_n(bday, 3)
        speech_output = "Die Mutter kann nicht " + abstr + " sondern erst nach dem Mutterschutz, also ab dem " + date2str(startkm) + " in Elternzeit gehen. Wieviel Monate möchte die Mutter in Elternzeit gehen?"
        reprompt_text = "Wieviel Monate möchte die Mutter nach dem " + date2str(startkm) + " in Elternzeit gehen?" 
        askmother = True
        session_attributes = {'iteration':106}
    elif seatit == 106:
        # 106
        speech_output = "Um Elterngeld zu erhalten muss der Vater mindestens 2 Lebensmonate Elternzeit nehmen. Von welchen Monat bis welchen Monat möchte der Vater in Elternzeit gehen?"
        reprompt_text = "Von welchen Monat bis welchen Monat möchte der Vater in Elternzeit gehen?" 
        askmother = False
        session_attributes = {'iteration':107}
    elif seatit == 107:
        # 107
        fmcount = fez['mcount']
        preText = "Der Vater kann maximal 12 "
        maxMonate = 12
        if 'mcount' in mez:
            mmcount = mez['mcount']
            if mmcount > 0:
                preText = "Die Mutter möchte " + str(mmcount) + " Monate Elterngeld beantragen. Der Vater kann dann nur noch " + str(12 - mmcount) 
            
        
        speech_output = preText + " Monate das volle Elterngeld beziehen. Sie haben " + str(fmcount) + " Monate gewählt. Von welchen Monat bis welchen Monat möchte der Vater in Elternzeit gehen?"
        reprompt_text = "Von welchen Monat bis welchen Monat möchte der Vater in Elternzeit gehen?" 
        askmother = False
        session_attributes = {'iteration':108}
    elif seatit == 108:
        bday = get_birthdate()
        mstart = mez['startkm']
        startkm = get_start_lifemonth_n(bday, mez['startkm'])
        endkm = get_end_lifemonth_n(bday, mez['endkm'])
        mcount = mez['mcount']
        
        fmstart = fez['startkm']
        fstartkm = get_start_lifemonth_n(bday, fez['startkm'])
        fendkm = get_end_lifemonth_n(bday, fez['endkm'])
        fmcount = fez['mcount']
        
        
        speech_output = "Sie haben insgesamt " + str(mcount + fmcount) + " Monate Elternzeit angegeben. Um das volle Basis Elterngeld zu erhalten, sollten beide Elternteile zusammen nicht mehr als 12 Monate Elterngeld nach dem Mutterschutz beantragen. "
        ## Test
        reprompt_text = speech_output
        session_attributes = {'iteration': 111}
        ## Ende Test
        if askmother:
            speech_output = speech_output + " Die Mutter wollte " + str(mez['mcount']) + " Monate nach dem Mutterschutz vom " + date2str(startkm) + " bis zum " + date2str(endkm) + " in Elternzeit gehen. Ist das OK oder "
            reprompt_text = "Von welchen Monat bis welchen Monat möchte die Mutter in Elternzeit gehen?" 
            speech_output = speech_output + reprompt_text
            session_attributes = {'iteration': 111}
        else:
            speech_output = speech_output + " Der Vater wollte " + str(fez['mcount']) + " Monate vom " + date2str(fstartkm) + " bis zum " + date2str(fendkm) + " in Elternzeit gehen. Ist das OK oder "
            reprompt_text = "Von welchen Monat bis welchen Monat möchte der Vater in Elternzeit gehen?" 
            speech_output = speech_output + reprompt_text
            session_attributes = {'iteration': 111}
    elif seatit == 109:
        if askmother:
            bday = get_birthdate()
            startm = get_start_lifemonth_n(bday, mez['startkm'])
            endm = get_end_lifemonth_n(bday, mez['endkm'])
            speech_output = " Die Mutter wollte " + str(mez['mcount']) + " Monate, vom " + date2str(startm) + " bis zum " + date2str(endm) + " in Elternzeit gehen. Ist das OK oder von welchem Monat bis welchem Monat möchte die Mutter in Elternzeit gehen?"
            reprompt_text = "Von welchen Monat bis welchen Monat möchte die Mutter in Elternzeit gehen?" 
        else:
            bday = get_birthdate()
            startf = get_start_lifemonth_n(bday, fez['startkm'])
            endf = get_end_lifemonth_n(bday, fez['endkm'])
            speech_output = " Der Vater wollte  " + str(fez['mcount']) + " Monate, vom " + date2str(startf) + " bis zum " + date2str(endf) + " in Elternzeit gehen. Ist das OK oder von welchem Monat bis welchem Monat möchte der Vater in Elternzeit gehen? "
            reprompt_text = "Von welchen Monat bis welchen Monat möchte der Vater in Elternzeit gehen?" 
        session_attributes = {'iteration': 111}
    elif seatit == 111:
        bday = get_birthdate()
        startf = get_start_lifemonth_n(bday, fez['startkm'])
        endf = get_end_lifemonth_n(bday, fez['endkm'])
        startm = get_start_lifemonth_n(bday, mez['startkm'])
        endm = get_end_lifemonth_n(bday, mez['endkm'])
        speech_output = "Ich werde " + str(mez['mcount']) + " Monate, vom " + months[startm.month-1] + " bis zum " +  months[endm.month-1] + " für die Mutter und " + str(fez['mcount']) + " Monate vom " +  months[startf.month-1] + " bis zum " + months[endf.month-1] + " für den Vater Elterngeld und Elternzeit beantragen. Ist das so OK?"
#        speech_output = "Ich werde " + str(mez['mcount']) + " Monate, vom " + date2str(startm) + " bis zum " + date2str(endm) + " für die Mutter und " + str(fez['mcount']) + " Monate vom " + date2str(startf) + " bis zum " + date2str(endf) + " für den Vater Elterngeld und Elternzeit beantragen. Ist das so OK?"
        reprompt_text = "Ich werde " + str(mez['mcount']) + " Monate für die Mutter und " + str(fez['mcount']) + " Monate für den Vater Elterngeld und Elternzeit beantragen. Ist das so OK?"
        session_attributes = {'iteration': 112}
    elif seatit != 100:
        session_attributes = {'iteration': 3}
        last_reprompt_text = " Möchte nur einer von Ihnen Elternzeit nehmen? "
        return norealise(session_attributes, card_title, speech_output, "")
    
    return build_response(session_attributes, build_speechlet_response(
        card_title, speech_output, reprompt_text, should_end_session))

        
def part_two(session,intent):
#   ========
    # Nur die Mutter möchte Elternzeit beantragen
    global askmother
    global last_reprompt_text
    global mez
    
    card_title = "part_two"
    should_end_session = False
    
    speech_output = "Fehler  " + str(session['attributes']['iteration']) + " in Teil zwei durch [" + intent['name'] + "]"
    reprompt_text = speech_output
    session_attributes = session['attributes']
    seatit = session_attributes['iteration']
    
    if seatit == 200:           
        speech_output = "Die meisten Mütter nehmen 10 Monate Elternzeit. Zusammen mit dem Mutterschutz ergibt das ein volles Jahr. Möchten Sie auch so Ihr Kind betreuen?"
        reprompt_text = "Möchte die Mutter insgesamt 1 Jahr das Kind betreuen?" 
        session_attributes = {'iteration':201}
    elif seatit == 201:           # "Nein"
        bday = get_birthdate()
        startkm = get_start_lifemonth_n(bday, 3)
        speech_output = "Die Mutter erhält Elterngeld erst nach dem Mutterschutz, also ab dem " + date2str(startkm) + ". Sie kann zwischen zwei und zehn Monaten Elternzeit beantragen. Von welchen Monat bis welchen Monat möchte die Mutter in Elternzeit gehen?"
        reprompt_text = "Von welchen Monat bis welchen Monat möchte die Mutter in Elternzeit gehen?" 
        askmother = True
        session_attributes = {'iteration':202}
    elif seatit == 202:
        bday = get_birthdate()
        mstart = mez['startkm']
        startkm = get_start_lifemonth_n(bday, mez['startkm'])
        endkm = get_end_lifemonth_n(bday, mez['endkm'])
        mcount = mez['mcount']
        
        if mcount < 2:
            # 203
            speech_output = "Um Elterngeld zu erhalten muss die Mutter nach dem Mutterschutz mindestens 2 Lebensmonate Elternzeit nehmen. Wieviel Monate möchte die Mutter in Elternzeit gehen?"
            reprompt_text = "Wieviel Monate möchte die Mutter in Elternzeit gehen?" 
            session_attributes = {'iteration':204}
        elif mcount > 10:
            # 204
            speech_output = "Sie haben  " + str(mcount) + " Monate ausgewählt. Um monatlich das volle Elterngeld zu erhalten, sollten Sie maximal 10 Monate Elternzeit beantragen. Von welchen Monat bis welchen Monat möchte die Mutter in Elternzeit gehen?"
            reprompt_text = "Von welchen Monat bis welchen Monat möchte die Mutter in Elternzeit gehen?" 
            session_attributes = {'iteration':205}
        elif mstart < 3:
            # 205
            abstr =  " ab dem ersten Lebensmonat, "
            if mstart == 1:
                abstr = " ab der Geburt,"
            startkm = get_start_lifemonth_n(bday, 3)
#            speech_output = "Die Mutter kann erst nach dem Mutterschutz, also ab dem " + date2str(startkm) + " in Elternzeit gehen. Wieviel Monate möchte die Mutter in Elternzeit gehen?"
            speech_output = "Die Mutter kann nicht " + abstr + " sondern erst nach dem Mutterschutz in Elternzeit gehen. Wieviel Monate möchte die Mutter nach dem " + date2str(startkm) + " in Elternzeit gehen?"
            reprompt_text = "Wieviel Monate möchte die Mutter nach dem " + date2str(startkm) + " in Elternzeit gehen?" 
            session_attributes = {'iteration':206}
        else:
            # 209
            speech_output = "OK, habe ich richtig verstanden, dass die Mutter " + str(mez['mcount']) + " Monate nach dem Mutterschutz vom " + months[startkm.month-1] + " bis zum " + months[endkm.month-1] + "  in Elternzeit gehen will?"
#            speech_output =  "Sind 55 Monate korrekt?"
            reprompt_text = "Sind " +  str(mez['mcount']) + " Monate, vom " + date2str(startkm) + " bis zum " + date2str(endkm) + " für die Mutter korrekt?"
            # reprompt_text =speech_output
            session_attributes = {'iteration':210}
    elif seatit == 209:
        bday = get_birthdate()
        mstart = mez['startkm']
        startkm = get_start_lifemonth_n(bday, mez['startkm'])
        endkm = get_end_lifemonth_n(bday, mez['endkm'])
        mcount = mez['mcount']
            
        speech_output = "OK, habe ich richtig verstanden, dass die Mutter " + str(mez['mcount']) + " Monate nach dem Mutterschutz vom " + date2str(startkm) + " bis zum " + date2str(endkm) + " in Elternzeit gehen will?"
#            speech_output =  "Sind 55 Monate korrekt?"
        reprompt_text = "Sind " +  str(mez['mcount']) + " Monate, vom " + date2str(startkm) + " bis zum " + date2str(endkm) + " für die Mutter korrekt?"
        session_attributes = {'iteration':210}        
        
    else:
        session_attributes = {'iteration': 3}
        last_reprompt_text = " Möchte nur eine von Ihnen Elternzeit nehmen? "
        return norealise(session_attributes, card_title, speech_output, "")
    
    return build_response(session_attributes, build_speechlet_response(
        card_title, speech_output, reprompt_text, should_end_session))

def part_three(session,intent):
#   ==========
    # Nur der Vater möchte Elternzeit beantragen
    global askmother
    global last_reprompt_text
    global askboth

    card_title = "part_three"
    should_end_session = False
    
    speech_output = "Fehler  " + str(session['attributes']['iteration']) + " in Teil drei durch [" + intent['name'] + "]"
    reprompt_text = speech_output
    session_attributes = session['attributes']
    seatit = session_attributes['iteration']
    
    if seatit == 300:           
        speech_output = "Viele Väter nehmen 12 Monate Elternzeit ab der Geburt. Möchten Sie Ihr Kind auch so betreuen?"
        reprompt_text = "Möchte der Vater 1 Jahr das Kind nach dem Mutterschutz betreuen?" 
        session_attributes = {'iteration':301}
    elif seatit == 301:           # "Nein
        fTime = 12
        if 'mcount' in mez:
            mcount = mez['mcount']
            fTime = fTime - mcount
        if fTime == 12:
            speech_output = "Der Vater kann ab der Geburt des Kindes in Elternzeit gehen. Um monatlich das volle Elterngeld zu erhalten, sollte der Vater zwischen zwei und 12 Monaten Elternzeit nehmen.  Von welchen Monat bis welchen Monat möchte der Vater in Elternzeit gehen?"
        elif fTime == 2:
            speech_output = "Der Vater kann ab der Geburt des Kindes in Elternzeit gehen. Die Mutter hat 10 Monate Elterngeld beantragt. Um das volle Elterngeld zu erhalten, kann der Vater noch zwei Lebensmonate Elternzeit nehmen.  Von welchen Monat bis welchen Monat möchte der Vater in Elternzeit gehen?"
        else:
            speech_output = "Der Vater kann ab der Geburt des Kindes in Elternzeit gehen. Die Mutter hat " + str(mcount) + " Monate Elterngeld beantragt. Um monatlich das volle Elterngeld zu erhalten, sollte der Vater zwischen zwei und " + str(12 - mcount) + " Monaten Elternzeit nehmen.  Von welchen Monat bis welchen Monat möchte der Vater in Elternzeit gehen?"

        reprompt_text = "Von welchen Monat bis welchen Monat möchte der Vater in Elternzeit gehen?" 
        askmother = False
        session_attributes = {'iteration':302}

    elif seatit == 302:
        session['attributes']['iteration'] = 800
        bday = get_birthdate()
        session['attributes']['iteration'] = 801
        mstart = fez['startkm']
        session['attributes']['iteration'] = 802
        startkm = get_start_lifemonth_n(bday, mstart)
        session['attributes']['iteration'] = 803
        endkm = get_end_lifemonth_n(bday, fez['endkm'])
        session['attributes']['iteration'] = 804
        mcount = fez['mcount']
        session['attributes']['iteration'] = 805

        if mcount < 2:
            # 303
            speech_output = "Um Elterngeld zu erhalten muss der Vater mindestens 2 Lebensmonate Elternzeit nehmen. Von welchen Monat bis welchen Monat möchte der Vater in Elternzeit gehen?"
            reprompt_text = "Von welchen Monat bis welchen Monat möchte der Vater in Elternzeit gehen?" 
            session_attributes = {'iteration':304}
        elif mcount > 12:
            # 304
            speech_output = "Um monatlich das volle Elterngeld zu erhalten sollte der Vater maximal 12 Monate Elterngeld beantragen. Sie haben  " + str(mcount) + " Monate ausgewählt.  Von welchen Monat bis welchen Monat möchte der Vater in Elternzeit gehen?"
            reprompt_text = "Von welchen Monat bis welchen Monat möchte der Vater in Elternzeit gehen?" 
            session_attributes = {'iteration':305}

        elif askboth:
            # 308 
            session['attributes']['iteration'] = 102
            return part_one(session, intent)
        else:
            # 308
            bday = get_birthdate()
            startkm = get_start_lifemonth_n(bday, fez['startkm'])
            endkm = get_end_lifemonth_n(bday, fez['endkm'])
        
            speech_output = "OK, habe ich richtig verstanden, dass der Vater " + str(fez['mcount']) + " Monate vom " + months[startkm.month-1] + " bis zum " + months[endkm.month-1] + " in Elternzeit gehen will?"
            reprompt_text = "Ist " +  months[startkm.month-1] + " bis " + months[endkm.month-1] + " für den Vater korrekt?"
            session_attributes = {'iteration':309}
    elif seatit == 308:
        bday = get_birthdate()
        mstart = fez['startkm']
        startkm = get_start_lifemonth_n(bday, fez['startkm'])
        endkm = get_end_lifemonth_n(bday, fez['endkm'])
        mcount = fez['mcount']
        
        speech_output = "OK, habe ich richtig verstanden, dass der Vater " + str(fez['mcount']) + " Monate vom " + months[startkm.month-1] + " bis zum " + months[endkm.month-1] + " in Elternzeit gehen will?"
#           speech_output = "OK, habe ich richtig verstanden, dass der Vater von " + intent["slots"]["start_month"]["value"] + " bis " + intent["slots"]["end_month"]["value"] + " in Elternzeit gehen will?"
        reprompt_text = "Ist " +  months[startkm.month-1] + " bis " + months[endkm.month-1] + " für den Vater korrekt?"
        session_attributes = {'iteration':309}
        
    else:
        session_attributes = {'iteration': 3}
        last_reprompt_text = " Möchte nur einer von Ihnen Elternzeit nehmen? "
        return norealise(session_attributes, card_title, speech_output, "") 
      
    return build_response(session_attributes, build_speechlet_response(
        card_title, speech_output, reprompt_text, should_end_session))
        
def part_four(session,intent):
#   =========
    global last_reprompt_text
    
    card_title = "part_four"
    should_end_session = False
    
    speech_output = "Fehler  " + str(session['attributes']['iteration']) + " in Teil vier durch [" + intent['name'] + "]"
    reprompt_text = speech_output
    session_attributes = session['attributes']
    seatit = session_attributes['iteration']

    if session['attributes']['iteration'] == 400:           # "Ja", korrekte Zusammenfassung bzw. Abkürzung
        speech_output = "Mit diesen Informationen kann ich die Elternzeit beim Arbeitgeber und das Elterngeld bei der Elterngeldstelle beantragen. Ist es OK, wenn ich alle anderen Personen bezogenen Daten aus ihrem Bürgerkonto für den Antrag auf Elterngeld verwende?"
        reprompt_text = "Ist es OK, wenn ich alle anderen Personen bezogenen Daten aus ihrem Bürgerkonto für den Antrag auf Elterngeld verwende?"
        # session_attributes = {'iteration':401}
        session_attributes = {'iteration':402}
    elif session['attributes']['iteration'] == 401:           # "Ja"
        speech_output = "Sind Sie damit einverstanden, dass die Elterngeldstelle von ihrem Arbeitgeber weitere Auskünfte einholt, soweit diese für die Entscheidung erforderlich sind?"
        reprompt_text = "Sind Sie damit einverstanden, dass die Elterngeldstelle von ihrem Arbeitgeber weitere Auskünfte einholt, soweit diese für die Entscheidung erforderlich sind?"
        session_attributes = {'iteration':402}
    elif session['attributes']['iteration'] == 402:           # "Ja" oder Nein
        speech_output = "OK, ich erstelle die Anträge, lege diese in ihrem Bürgerkonto ab. Ich beantrage zuerst Elternzeit und danach Elterngeld und melde mich, wenn der Bescheid im Postfach eingegangen ist. Bis bald. "
        session_attributes = {'iteration':403}
        should_end_session = True
    elif session['attributes']['iteration'] == 403:           # "Ja"
        speech_output = "Mit den vorhandenen Informationen habe ich die Anträge ausgefüllt und Ihnen soeben per E-Mail zugesendet. Sie müssen diese vervollständigen, zuerst die Elternzeit beim Arbeitgeber und dann das Elterngeld bei der Elterngeldstelle beantragen. Viel Erfolg. "
        session_attributes = {'iteration':404}
        should_end_session = True    
    elif session['attributes']['iteration'] == 404:
        speech_output = "Ohne weitere Informationen kann ich Ihnen beim Erstellen der Anträge leider nicht helfen. Sie müssen Elternzeit beim Arbeitgeber und Elterngeld bei der Elterngeldstelle des Jugendamts in Ihrem Wohnbezirk beantragen. Die Beantragung geht auch über das Bürgeramt im Internet. Viel Erfolg. "
        session_attributes = {'iteration':405}
        should_end_session = True
    elif session['attributes']['iteration'] == 405:           # "Ja"
        speech_output = "Genauer kann ich ihnen das nicht erklären. Sie können im Internet unter dem Stichwort Elterngeld viele Informationen dazu finden. Wenn Sie genügend Informationen zu dem Zusammenhang zwischen Elternzeit, Lebensmonate und Elterngeld gesammelt haben, kann ich beim Erstellen der Anträge helfen. Hoffentlich bis bald. "
        session_attributes = {'iteration':406}
        should_end_session = True
    elif seatit == 406:
        speech_output = "Ich würde Ihre Personendaten, die Geburtsbescheinigung des neugeborenen Kindes, Ihr Einkommen, Ihre Kontoverbindung und die Daten von Ihren Arbeitgebern verwenden. Ist das OK?"
        reprompt_text = "Ist es OK, das ich diese Daten verwende?"
        session_attributes = {'iteration':407}
    elif seatit == 407:
        speech_output = "Die benötigten Personendaten sind Ihre Namen, die Wohnadresse, Ihre Personalausweiskopien, Ihre Staatsangehörigkeit, der Familienstand, Ihre Krankenversicherung, Ihre Kontoverbindung für die Überweisung des Elterngeldes und Ihre Kontaktdaten. Ist es OK, das ich diese Daten verwende?"
        reprompt_text = "Ist es OK, das ich diese Daten verwende?"
        session_attributes = {'iteration':408}
    elif seatit == 408:
        speech_output = "Die benötigten Daten für das Elterngeld ist das zu versteuernde Gesamteinkommen vom Vorjahr, Ihre letzten Gehaltsbescheinigungen und Ihre Krankenversicherung. Ist es OK, das ich diese Daten verwende?"
        reprompt_text = "Ist es OK, das ich diese Daten verwende?"
        session_attributes = {'iteration':409}
    elif seatit == 409:
        speech_output = "Zur Überweisung des Elterngeldes wird Ihre Kontoverbindung benötigt. Ist es OK, das ich diese Daten verwende?"
        reprompt_text = "Ist es OK, das ich diese Daten verwende?"
        session_attributes = {'iteration':410}
    elif seatit == 410:
        speech_output = "Zur Beantragung des Elterngeldes benötige ich die Bestätigung Ihres Arbeitgebers zur Elternzeit, den Nachweis zum Mutterschaftsgeld und die Anschriften der Arbeitgeber. Ist es OK, das ich diese Daten verwende?"
        reprompt_text = "Ist es OK, das ich diese Daten verwende?"
        session_attributes = {'iteration':411}
    elif seatit == 411:
        speech_output = "Ich würde Ihre Namen und die Wohnadresse, die Geburtsbescheinigung des neugeborenen Kindes, Ihre Personalausweiskopien, die Staatsangehörigkeit, den Familienstand, das zu versteuernde Gesamteinkommen vom Vorjahr, Ihre Krankenversicherung, Ihre Kontoverbindung für die Überweisung des Elterngeldes, Ihre letzte Gehaltsbescheinigungen, die Bestätigung Ihres Arbeitgebers zur Elternzeit, den Nachweis zum Mutterschaftsgeld, die Anschrift der Arbeitgeber und für Rückfragen Ihre Kontaktdaten verwenden. Ist es OK, das ich diese Daten verwende?"
        reprompt_text = "Ist es OK, das ich diese Daten verwende?"
        session_attributes = {'iteration':412}
    else:
        session_attributes = {'iteration': 3}
        last_reprompt_text = " Möchte nur einer von Ihnen Elternzeit nehmen? "
        return norealise(session_attributes, card_title, speech_output, "")
    if should_end_session:
        clearDB(session)
    return build_response(session_attributes, build_speechlet_response(
        card_title, speech_output, reprompt_text, should_end_session))
#---------------------------------------------------------------
# -------- intents handler -------------------------------------
#---------------------------------------------------------------
def agreeing(session,intent): 
#   ========
    global askmother
    
    card_title = "agreeing"
    should_end_session = True
    
    speech_output = "Fehler  " + str(session['attributes']['iteration']) + " in der Antwort Einverstanden "
    reprompt_text = speech_output
    session_attributes = session['attributes']
    seatit = session['attributes']['iteration']
    if seatit == 0:
        return part_start(session, intent)
    elif seatit == 6 or seatit == 7:
        session['attributes']['iteration'] = 1
        return part_start(session, intent)
    elif seatit == 1 or seatit == 3:      # "Ja", "Verstanden"
        return part_start(session, intent)
    elif seatit == 8:   #  "ja", "Verstanden"
        session['attributes']['iteration'] = 1
        return part_start(session, intent)
    elif seatit == 111:
        if askmother:
            askmother = False
            session['attributes']['iteration'] = 1
            return part_start(session, intent)
        else:
            askmother = True
            session['attributes']['iteration'] = 109
            return part_one(session, intent)
    elif seatit == 101 or seatit == 112:
        session['attributes']['iteration'] = 400
        return part_four(session, intent)
    elif seatit == 201:
        session['attributes']['iteration'] = 400
        return part_four(session, intent)
    elif seatit == 210 and askboth:
        session['attributes']['iteration'] = 301
        return part_three(session, intent)
    elif seatit == 210 or seatit == 301 or seatit == 309:
        session['attributes']['iteration'] = 400  
        return part_four(session, intent)
    elif seatit == 307:
        return part_three(session, intent)
    elif seatit == 401 or seatit == 402:
        return part_four(session, intent)
    elif seatit == 407 or seatit == 408 or seatit == 409 or seatit == 410 or seatit == 411 or seatit == 412:
        # session['attributes']['iteration'] = 401
        session['attributes']['iteration'] = 402
        return part_four(session, intent)
    else:
        return norealise(session_attributes, card_title, speech_output, "")
    
    return build_response(session_attributes, build_speechlet_response(
        card_title, speech_output, None, should_end_session))

def disagreeing(session,intent): 
#   ===========    
    card_title = "disagreeing"
    should_end_session = True
    
    speech_output = "Fehler  " + str(session['attributes']['iteration']) + " in der Antwort garnicht Einverstanden. "
    reprompt_text = speech_output
    session_attributes = session['attributes']
    seatit = session['attributes']['iteration']
    
    if seatit == 0:
        session['attributes']['iteration'] = 1
        return part_start(session, intent)
    elif seatit == 6 or seatit == 7:      # "Nein" (~ nicht einverstanden)
        session['attributes']['iteration'] = 404
        return part_four(session, intent)
    elif seatit == 1:   #  # "Nein" (~ nicht verstanden)
        session['attributes']['iteration'] = 7
        return part_start(session, intent)
    elif seatit == 3:
        session['attributes']['iteration'] = 100
        return past_one(session,intent)
    elif seatit == 8:      # nochmal nicht verstanden
        session['attributes']['iteration'] = 405
        return part_four(session, intent)
    elif seatit == 101:
        session['attributes']['iteration'] = 201
        return part_two( session, intent)
    elif seatit == 111:
        session['attributes']['iteration'] = 1
        return part_start(session, intent)
    elif seatit == 112:
        session['attributes']['iteration'] = 109
        askmother = False
        return part_one(session, intent)
    elif seatit == 201:      # nochmal nicht verstanden
        return part_two(session, intent)
    elif seatit == 210:      # zusammenfassung nicht ok
        # session['attributes']['iteration'] = 207 # noch nicht implementiert
        session['attributes']['iteration'] = 201
        return part_two(session, intent)
    elif seatit ==301:
        return part_three(session, intent)
    elif seatit == 309:
        session['attributes']['iteration'] = 301
        return part_three(session, intent)
    # elif seatit == 401:      # keine infos aus dem Bürgerkonto
    elif seatit == 402:      # keine infos aus dem Bürgerkonto
        session['attributes']['iteration'] = 403
        return part_four(session, intent)
    elif seatit == 407 or seatit == 408 or seatit == 409 or seatit == 410 or seatit == 411 or seatit == 412:
        session['attributes']['iteration'] = 403
        return part_four(session, intent)
    elif seatit == 402:      # wird nicht ausgewertet
        return part_four(session, intent)
    else:
        return norealise(session_attributes, card_title, speech_output, "")
        
    return build_response(session_attributes, build_speechlet_response(
        card_title, speech_output, "", should_end_session))
        

def how_much(session,intent): 
#   ========    
    card_title = "how_much"
    should_end_session = True
    speech_output = "Fehler  " + str(session['attributes']['iteration']) + " in der Antwort wie viel Elterngeld "
    reprompt_text = speech_output
    session_attributes = session['attributes']
    seatit = session['attributes']['iteration']
    
    if seatit == 0:     
        session['attributes']['iteration'] = 5
        return part_start(session, intent)
    else:
        return norealise(session_attributes, card_title, speech_output, "")

def one(session,intent): 
#   ===    
    card_title = "one"
    should_end_session = True
    speech_output = "Fehler  " + str(session['attributes']['iteration']) + " in der Antwort nur Einer "
    reprompt_text = speech_output
    session_attributes = session['attributes']
    seatit = session['attributes']['iteration']
    
    if seatit == 2 or seatit == 3:     
        return part_start(session, intent)
    else:
        return norealise(session_attributes, card_title, speech_output, "")
    
def further_information(session,intent): 
#   ===================    
    card_title = "further_information"
    should_end_session = True
    speech_output = "Fehler  " + str(session['attributes']['iteration']) + " in der Antwort welche Informationen "
    reprompt_text = speech_output
    session_attributes = session['attributes']
    seatit = session['attributes']['iteration']
    
    if seatit == 0 or seatit == 6: 
        return part_start(session, intent)
    # elif seatit == 401:
    elif seatit == 402:
        session['attributes']['iteration'] = 406
        return part_four(session, intent)
    elif seatit == 406 or seatit == 407 or seatit == 408 or seatit == 409 or seatit == 410 or seatit == 411 or seatit == 412:     
        session['attributes']['iteration'] = 411
        return part_four(session, intent)
    else:
        return norealise(session_attributes, card_title, speech_output, "")

def both(session,intent): 
#   ====    
    card_title = "both"
    should_end_session = True
    speech_output = "Fehler  " + str(session['attributes']['iteration']) + " in der Antwort Beide "
    reprompt_text = speech_output
    session_attributes = session['attributes']
    seatit = session['attributes']['iteration']
    
    if seatit == 2 or seatit == 3:     
        session['attributes']['iteration'] = 100
        return part_one(session, intent)
    else:
        return norealise(session_attributes, card_title, speech_output, "")
    
def mother(session,intent): 
#   ======
    global last_reprompt_text
    
    card_title = "mother"
    should_end_session = False
    speech_output = "Fehler  " + str(session['attributes']['iteration']) + " in der Antwort Mutter. "
    reprompt_text = speech_output
    session_attributes = session['attributes']
    seatit = session['attributes']['iteration']
    
    if seatit == 2 or seatit == 3 or seatit == 4:     
        session['attributes']['iteration'] = 200
        return part_two(session, intent)
    else:
        return norealise(session_attributes, card_title, speech_output, "")

def father(session,intent): 
#   ======   
    card_title = "father"
    should_end_session = True
    speech_output = "Fehler  " + str(session['attributes']['iteration']) + " in der Antwort Vater "
    reprompt_text = speech_output
    session_attributes = session['attributes']
    seatit = session['attributes']['iteration']
    
    if seatit == 2 or seatit == 3 or seatit == 4:     
        session['attributes']['iteration'] = 300
        return part_three(session, intent)
    else:
        return norealise(session_attributes, card_title, speech_output, "")

def m_count(session,intent): 
#   =======
    global mez, fez
    
    card_title = "m_count"
    should_end_session = True
    speech_output = "Fehler  " + str(session['attributes']['iteration']) + " in der Antwort Anzahl Monate "
    reprompt_text = speech_output
    session_attributes = session['attributes']
    seatit = session['attributes']['iteration']
    
    mcount = int(intent['slots']['duration']['value'])
    if askmother:
        mez = {'startkm': 3, 'endkm': mcount+2, 'mcount': mcount}

        # bday = get_birthdate()
        # startkm = get_start_lifemonth_n(bday, mez['startkm'])
        # endkm = get_end_lifemonth_n(bday, mez['endkm'])
    
        # speech_output = speech_output + str(mcount) + " Monate. " +  "OK, habe ich richtig verstanden, dass die Mutter nach dem Mutterschutz in Elternzeit vom " + date2str(startkm) + " bis zum " + date2str(endkm) + " gehen will?"
        if seatit == 104 or seatit == 105 or seatit == 107 or seatit == 108 or seatit == 106:
            session['attributes']['iteration'] = 102
            return part_one(session, intent)
        elif seatit == 202:     
            return part_two(session, intent)
        elif seatit == 206 or seatit == 204:
            session['attributes']['iteration'] = 202
            return part_two(session, intent)
        else:
            return norealise(session_attributes, card_title, speech_output, "")
    else:
        fez =  {'startkm': 1, 'endkm': mcount, 'mcount': mcount}
        
        return norealise(session_attributes, card_title, speech_output, "")

def m_anf_end(session,intent): 
#   =========
    global fez
    global mez
    global askmother
    
    card_title = "m_anf_end"
    should_end_session = True
    speech_output = "Fehler  " + str(session['attributes']['iteration']) + " in der Antwort von bis Monate "
    if askmother:
        speech_output = speech_output + " bei Frage Mutter. "
    else:
        speech_output = speech_output + " bei Frage Vater. "
    reprompt_text = speech_output
    session_attributes = session['attributes']
    seatit = session['attributes']['iteration']

    startmstr = intent["slots"]["start_month"]["value"]
    endmstr =  intent["slots"]["end_month"]["value"]
    err = False
    err_txt = ""
    if startmstr in months2n:
        startm = months2n[startmstr]
    else:
        err = True
        err_txt =  " " + startmstr + " ist kein Monatsname. "
    if endmstr in months2n:
        endm = months2n[endmstr]
    else:
        err = True
        if err_txt == "":
            err_txt =  " " + endmstr + " ist kein Monatsname. "
        else:
            err_txt = " " + startmstr + " und " + endmstr + " sind keine Monatsnamen. "
        
    startkm = -1
    endkm = -1
    bday = get_birthdate()
    
    if err == False:
        for i in range(1, 15):
            if get_start_lifemonth_n(bday, i).month == startm:
                startkm = i
                break
        for j in range(1, 27):
            if get_end_lifemonth_n(bday, j).month == endm:
                if j >= startkm:
                    endkm = j
                    break  
   
        err = startkm == -1 or endkm == -1
        if startkm == -1:
            err_txt =  " Beim Endemonat " + str(endm) + " ist " + str(startm) + " kein Anfang eines Lebensmoats. "
        if endkm == -1:
            err_txt =  " Beim Anfangsmonat " + str(startm) + " ist " + str(endm) + " kein Ende eines Lebensmonats. "
    
    if err == False:
        mcount = endkm - startkm +1
        if askmother:
            mez = {'startkm':startkm, 'endkm':endkm, 'mcount':mcount}
            if seatit == 104 or seatit == 105 or seatit == 106 or seatit == 107 or seatit == 108 or seatit == 111:
                askmother = False
                session['attributes']['iteration'] = 102
                return part_one(session, intent)
            elif seatit == 202 or seatit == 204 or seatit == 205 or seatit == 206:
                session['attributes']['iteration'] = 202
                return part_two(session, intent)
        else:
            fez = {'startkm':startkm, 'endkm':endkm, 'mcount':mcount}
            if seatit == 104 or seatit == 105 or seatit == 106 or seatit == 107 or seatit == 108:
                session['attributes']['iteration'] = 102
                return part_one( session, intent)
            elif seatit == 111:
                askmother = not askmother
                session['attributes']['iteration'] = 102
                return part_one( session, intent)
            elif seatit == 302 or seatit == 304 or seatit == 305:
                session['attributes']['iteration'] = 302
                return part_three(session, intent)
    
    return norealise(session_attributes, card_title, speech_output, err_txt)

def m_anf_count(session,intent): 
#   ===========
    global fez
    global mez
    global askmother
    
    card_title = "m_anf_count"
    should_end_session = True
    speech_output = "Fehler  " + str(session['attributes']['iteration']) + " in der Antwort von Monat n Monate "
    if askmother:
        speech_output = speech_output + " bei Frage Mutter. "
    else:
        speech_output = speech_output + " bei Frage Vater. "
    reprompt_text = speech_output
    session_attributes = session['attributes']
    seatit = session['attributes']['iteration']

    startmstr = intent["slots"]["start_month"]["value"]
    mcount = int(intent['slots']['duration']['value'])

    err = False
    err_txt = ""
    if startmstr in months2n:
        startm = months2n[startmstr]
    else:
        err = True
        err_txt =  " " + startmstr + " ist kein Monatsname. "
        
    startkm = -1
    bday = get_birthdate()
    
    if err == False:
        for i in range(1, 15):
            if get_start_lifemonth_n(bday, i).month == startm:
                startkm = i
                break
   
        err = startkm == -1 
        if startkm == -1:
            err_txt =  " " + str(startm) + " ist kein Anfang eines Lebensmoats. "
 
    if err == False:
        
        endkm = startkm + mcount -1
        
        if askmother:
            mez = {'startkm':startkm, 'endkm':endkm, 'mcount':mcount}
            if seatit == 104 or seatit == 105 or seatit == 107 or seatit == 108 or seatit == 111:
                askmother = False
                session['attributes']['iteration'] = 102
                return part_one(session, intent)
            elif seatit == 202 or seatit == 205:
                session['attributes']['iteration'] = 202
                return part_two(session, intent)
        else:
            fez = {'startkm':startkm, 'endkm':endkm, 'mcount':mcount}
            if seatit == 104 or seatit == 105 or seatit == 107 or seatit == 108:
                session['attributes']['iteration'] = 102
                return part_one( session, intent)
            elif seatit == 111:
                askmother = not askmother
                session['attributes']['iteration'] = 102
                return part_one( session, intent)
            elif seatit == 302 or seatit == 304 or seatit == 305:
                session['attributes']['iteration'] = 302
                return part_three(session, intent)
    
    return norealise(session_attributes, card_title, speech_output, err_txt)

def personal_data(session,intent): 
#   =============    
    card_title = "personal_data"
    should_end_session = True
    speech_output = "Fehler  " + str(session['attributes']['iteration']) + " in der Antwort Personendaten. "
    reprompt_text = speech_output
    session_attributes = session['attributes']
    seatit = session['attributes']['iteration']
    if seatit == 0 or seatit == 6:     
        # session['attributes']['iteration'] = 6
        return part_start(session, intent)
    # elif seatit == 401:
    elif seatit == 402:
        session['attributes']['iteration'] = 406
        return part_four(session, intent)
    elif seatit == 406 or seatit == 407 or seatit == 408 or seatit == 409 or seatit == 410 or seatit == 411 or seatit == 412:     
        session['attributes']['iteration'] = 407
        return part_four(session, intent)
    else:
        return norealise(session_attributes, card_title, speech_output, "")

def earnings(session,intent): 
#   ========    
    card_title = "earnings"
    should_end_session = True
    speech_output = "Fehler  " + str(session['attributes']['iteration']) + " in der Antwort Einkommen. "
    reprompt_text = speech_output
    session_attributes = session['attributes']
    seatit = session['attributes']['iteration']
    
    if seatit == 407 or seatit == 408 or seatit == 409 or seatit == 410 or seatit == 411 or seatit == 412:     
        session['attributes']['iteration'] = 408
        return part_four(session, intent)
    else:
        return norealise(session_attributes, card_title, speech_output, "")

def account_relationship(session,intent): 
#   ====================    
    card_title = "account_relationship"
    should_end_session = True
    speech_output = "Fehler  " + str(session['attributes']['iteration']) + " in der Antwort Kontoverbindung. "
    reprompt_text = speech_output
    session_attributes = session['attributes']
    seatit = session['attributes']['iteration']
    
    if seatit == 407 or seatit == 408 or seatit == 409 or seatit == 410 or seatit == 411 or seatit == 412:     
        session['attributes']['iteration'] = 409
        return part_four(session, intent)
    else:
        return norealise(session_attributes, card_title, speech_output, "")

def employer(session,intent): 
#   ========    
    card_title = "employer"
    should_end_session = True
    speech_output = "Fehler  " + str(session['attributes']['iteration']) + " in der Antwort Arbeitgeber. "
    reprompt_text = speech_output
    session_attributes = session['attributes']
    seatit = session['attributes']['iteration']
    
    if seatit == 407 or seatit == 408 or seatit == 409 or seatit == 410 or seatit == 411 or seatit == 412:     
        session['attributes']['iteration'] = 410
        return part_four(session, intent)
    else:
        return norealise(session_attributes, card_title, speech_output, "")

def repeat(session,intent): 
#   ======    
    card_title = "repeat"
    should_end_session = True
    speech_output = "Fehler  " + str(session['attributes']['iteration']) + " in der Antwort Wiederholen. "
    reprompt_text = speech_output
    session_attributes = session['attributes']
    seatit = session['attributes']['iteration']
    
    if seatit == 1 or seatit == 2 or seatit == 3 or seatit == 4 or seatit == 6 or seatit == 7 or seatit == 8:
        session['attributes']['iteration'] = seatit - 1
        return part_start(session, intent)
    elif seatit == 0:
        session['attributes']['iteration'] = 5
        return part_start(session, intent)
    elif seatit == 101 or seatit == 104 or seatit == 105 or seatit == 106 or seatit == 107 or seatit == 108 or seatit == 112:
        session['attributes']['iteration'] = seatit - 1
        return part_one(session, intent)
    elif seatit == 111:
        session['attributes']['iteration'] = 109
        return part_one(session, intent)
    elif seatit == 201 or seatit == 202 or seatit == 204 or seatit == 205 or seatit == 206 or seatit == 210:
        session['attributes']['iteration'] = seatit - 1
        return part_two(session, intent)
    elif seatit == 301 or seatit == 302 or seatit == 304 or seatit == 305 or seatit == 309:
        session['attributes']['iteration'] = seatit - 1
        return part_three(session, intent)
    elif seatit == 401 or seatit == 407 or seatit == 408 or seatit == 409 or seatit == 410 or seatit == 411 or seatit == 412:     
        session['attributes']['iteration'] = seatit - 1
        return part_four(session, intent)
    elif seatit == 402:
        session['attributes']['iteration'] = 400
        return part_four(session, intent)
    else:
        return norealise(session_attributes, card_title, speech_output, "")

def pause_intent(session, intent):
    global last_reprompt_text
    card_title = "Pause"
    speech_output = "OK. Pause. Bis später. " 
    # Setting this to true ends the session and exits the skill.
    
    user= session['user']['userId']  # get the userId of the current user
    user=user[18:] # we need the string after the 19th character.

    table = client.Table('elterngeld') # DynamoDB Table Name

    seatit = session['attributes']['iteration']
    # table.put_item(Item={'account':"testaccount01",'iterationtst':"tst01"})
    table.put_item(Item={'account':user,'iteration':seatit, 'lasttext': last_reprompt_text, 'mez': mez, 'fez': fez, 'askboth': askboth, 'askmother': askmother})

    reprompt_text = speech_output
    session_attributes = session['attributes']
    should_end_session = True
    
    return build_response(session_attributes, build_speechlet_response(
        card_title, speech_output, reprompt_text, should_end_session))

## ---------------------------------------------------------
## ---------------- End Intents Handler --------------------
## ---------------------------------------------------------
    
def handle_session_end_request(session, intent):
    card_title = "Session Ended"
    speech_output = "Ich beende meine Befragung. Ich wünsche ihnen noch einen schönen Tag!" 
    # Setting this to true ends the session and exits the skill.
    should_end_session = True
    reprompt_text = speech_output
    session_attributes = session['attributes']
    
    clearDB(session)
    
    return build_response(session_attributes, build_speechlet_response(
        card_title, speech_output, reprompt_text, should_end_session))

# --------------- Events ------------------

def on_session_started(session_started_request, session):
    """ Called when the session starts.
        One possible use of this function is to initialize specific 
        variables from a previous state stored in an external database
    """
    # Add additional code here as needed
    pass

    

def on_launch(launch_request, session):
    # Dispatch to your skill's launch message
    return welcome(session)


def on_intent(intent_request, session):

    intent = intent_request['intent']
    intent_name = intent_request['intent']['name']

    # Dispatch to your skill's intent handlers

    if intent_name == "agreeing":
        return agreeing(session,intent)
    elif intent_name == "disagreeing":
        return disagreeing(session,intent)
    elif intent_name == "how_much":
        return how_much(session,intent)
    elif intent_name == "further_information":
        return further_information(session,intent)
    elif intent_name == "one_person":
       return one(session,intent)
    elif intent_name == "both":
        return both(session,intent)
    elif intent_name == "mother":
        return mother(session,intent)     
    elif intent_name == "father":
        return father(session,intent)
    elif intent_name == "m_count":
        return m_count(session, intent)
    elif intent_name == "m_anf_end":
        return m_anf_end(session, intent)
    elif intent_name == "m_anf_count":
        return m_anf_count(session, intent)
    elif intent_name == "personal_data":
        return personal_data(session, intent)
    elif intent_name == "earnings":
        return earnings(session, intent)
    elif intent_name == "account_relationship":
        return account_relationship(session, intent)
    elif intent_name == "employer":
        return employer(session, intent)
    elif intent_name == "repeat":
        return repeat(session, intent)
    elif intent_name == "AMAZON.HelpIntent":
        return repeat(session, intent)
#        return get_welcome_response()
    elif intent_name == "AMAZON.PauseIntent":
        return pause_intent(session, intent)
    elif intent_name == "AMAZON.CancelIntent" or intent_name == "AMAZON.StopIntent":
        return handle_session_end_request(session, intent)
    
    card_title = "on_intent"
    speech_output = "Fehler  " + str(session['attributes']['iteration']) + " in unbekannter Intent-Name [" + intent_name + "]"
    reprompt_text = speech_output
    session_attributes = session['attributes']
    should_end_session = True
    
    return build_response(session_attributes, build_speechlet_response(
        card_title, speech_output, reprompt_text, should_end_session))


def on_session_ended(session_ended_request, session):

#    print("on_session_ended requestId=" + session_ended_request['requestId'] +
#         ", sessionId=" + session['sessionId'])
    card_title = "Session Ended"
    speech_output = "Ich beende die Sitzung. Ich wünsche ihnen noch einen schönen Tag!" 
    # Setting this to true ends the session and exits the skill.
    should_end_session = True
    reprompt_text = speech_output
    session_attributes = session['attributes']
    
    clearDB(session)
    
    return build_response(session_attributes, build_speechlet_response(
        card_title, speech_output, reprompt_text, should_end_session))
    


# --------------- Main handler ------------------

def lambda_handler(event, context):
    """ Route the incoming request based on type (LaunchRequest, IntentRequest,
    etc.) The JSON body of the request is provided in the event parameter.
    """
    print("Incoming request...")



    if event['session']['new']:
        on_session_started({'requestId': event['request']['requestId']},
                           event['session'])

    if event['request']['type'] == "LaunchRequest":
        #wilcome -Um Elterngeld vom Staat ...
        return on_launch(event['request'], event['session'])
    elif event['request']['type'] == "IntentRequest":
        return on_intent(event['request'], event['session'])
    elif event['request']['type'] == "SessionEndedRequest":
        return on_session_ended(event['request'], event['session'])
    
    session_attributes = event['session']['attributes']
    return build_response(session_attributes, build_speechlet_response(
        "lambda_handler", "Lambda Händler mit Typ " + event['request']['type'], "Lambda Händler mit Typ " + event['request']['type'], True))
